package com.jiayue.simulation.ice102;

import com.jiayue.simulation.ice102.codec.Encoder;
import com.jiayue.simulation.ice102.codec.Decoder;
import com.jiayue.simulation.ice102.handler.ClientHandler;
import io.netty.bootstrap.Bootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.timeout.IdleStateHandler;

import java.util.concurrent.TimeUnit;

public class Client implements ConsoleInterface {

    private String ip;

    private Integer port;


    public Client(String ip,Integer port) {
        this.ip = ip;
        this.port = port;

        start();
    }

    @Override
    public void start() {
        //客户端创建一个事件组
        EventLoopGroup eventExecutors = new NioEventLoopGroup();

        try {
            //创建一个客户端启动对象
            Bootstrap bootstrap = new Bootstrap();
            bootstrap.group(eventExecutors)
                    .channel(NioSocketChannel.class)
                    .handler(new ChannelInitializer<SocketChannel>() {

                        @Override
                        protected void initChannel(SocketChannel socketChannel) throws Exception {
                            socketChannel.pipeline().addLast(new IdleStateHandler(
                                    60,
                                    0,
                                    0,
                                    TimeUnit.SECONDS));
                            //添加编解码
                            socketChannel.pipeline().addLast("decoder", new Encoder());
                            socketChannel.pipeline().addLast("encoder", new Decoder());
                            socketChannel.pipeline().addLast(new ClientHandler());
                        }
                    });

            //启动连接
            ChannelFuture channelFuture = bootstrap.connect(ip, port);

            channelFuture.addListener(new ChannelFutureListener() {
                //使用匿名内部类，ChannelFutureListener接口
                //重写operationComplete方法
                @Override
                public void operationComplete(ChannelFuture future) throws Exception {
                    //判断是否操作成功
                    if (future.isSuccess()) {
                        System.out.println("客户端连接成功");
                    } else {

                        System.out.println("客户端连接失败");
                    }
                }
            });

            //监听关闭通道操作
            channelFuture.channel().closeFuture().sync();

        }catch (Exception e){
            e.printStackTrace();
        }


    }
}
